<%--
 * Component: cases
 *
 * Description:
 *   Merges two or more studies into one new study
 *
 * Parameters:
 *
 * @param cBack         URL to go back after splitting the study
 * @param cGr           List of Study Instance UIDs to be merged
 * @param cOp           Merge operation (Confirm | Merge)
 * @param cPrimary      Study Instance UID of the Primary Study
 * @param cKeepReports  Keep all reports or Primary Study's report only
 * @param cContext      To get fields to be displayed in the studies table
 *
 * The patient and study information in the Primary Study gets copied into the other studies.
 *
 * If cKeepReports is "true" then reports from other studies are attached as addendums to the
 * Primary Study's report.
 *
 * If cKeepReports is not "true" then reports from other studies are purged
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
--%><%@
	page import="java.util.*, imagemed.dotcom.*, imagemed.imsession.*, imagemed.util.*, imagemed.jsputil.*,
			imagemed.exception.*, imagemed.tables.*, imagemed.types.flag.IMTypeur, java.text.Format"
	
%><%@
	page errorPage="../main/exceptionHandler.jsp"

%><%!
	static String colBodyBg;
	static String colBodyText;
	static String colBodyLink;
	static String colBodyVlink;

	static {
		imagemed.tables.TableProperties tp = new imagemed.tables.TableProperties();
		colBodyBg = tp.getColor( "BodyBg" );
		colBodyText = tp.getColor( "BodyText" );
		colBodyLink = tp.getColor( "BodyLink" );
		colBodyVlink = tp.getColor( "BodyVlink" );
	}

	/** Show this page (end error messages) under this tab */
	static final String m_tab = "cases/index.jsp";

	/** Link to the 'Group Orders' page */
	static final String m_groupPageLink =
			"<A href=\"/cases/orders.jsp\" onMouseOver=\"window.status='Group Orders'; return true\">" + LocalizedMessage.getSimpleMessage( "tabGroupOrders" ) + "</A>";

	/** Link to the 'Corrections' page */
	static final String m_correctionsPageLink =
			"<A href=\"/qc/index.jsp\" onMouseOver=\"window.status='Corrections'; return true\">" + LocalizedMessage.getSimpleMessage( "tabCorrections" ) + "</A>";

%><%

	IMSession is = new IMSession( request, response, out );

	is.response.setHeader( "pragma", "no-cache" );
	is.response.setHeader( "Cache-Control", "no-cache, must-revalidate" );
	is.response.setHeader( "Expiry-Date", "Friday, 31-Dec-70 23:59:59 GMT" );

	if ( !is.regOK() )
		return;

	String parIface = request.getParameter( "cIface" );
	boolean isIface = false;
	String tab = m_tab;

	if ( "tabs".equals( parIface ) ) {
		// show tabs for page called via HTTP API
		isIface = true;
	}
	else if ( "true".equals( parIface ) ) {
		// hide tabs for page called via HTTP API
		isIface = true;
		tab = null;		
	}
	
	String back = request.getParameter( "cBack" );
	if ( back == null && !isIface )
		back = request.getHeader( "REFERER" );

	if ( !is.hasRight( IMTypeur.EDIT_RIGHT ) ) { // neither Edit nor Admin right
		Format[] msgFormat = new Format[] { null };
		Object[] msgObject = new Object[] { is.regID };
		throw new IMCommonGUIException( is,
				LocalizedMessage.getCompoundMessage( "EXCEPTION_CASES_MERGE_NO_RIGHTS", msgFormat, msgObject, true),
				tab, LocalizedMessage.getSimpleMessage( "EXCEPTION_CASES_ORDERS_MERGE_TITLE" ), back, 0 );
	}

	Vector<String> studies = new Vector<String>();
	String parGr[] = request.getParameterValues( "cGr" );
	if ( parGr == null || parGr.length < 2 )
		throw new IMCommonGUIException( is, LocalizedMessage.getSimpleMessage( "EXCEPTION_CASES_MERGE_AT_LEAST_TWO" ), tab, LocalizedMessage.getSimpleMessage( "EXCEPTION_CASES_ORDERS_MERGE_TITLE" ), back, 0 );

	boolean hasOrders = false;
	for ( int i = 0; i < parGr.length; i++ ) {
		for ( StringTokenizer st = new StringTokenizer( parGr[ i ] ); st.hasMoreTokens(); ) {
			String study = st.nextToken();
			studies.add( study );
			
			if ( StudyToolbox.isOrder( study ) ) {
				hasOrders = true;
			}
		}
	}

	if ( ! hasOrders ) {
		// Merging studies - can only be performed if both studies has the same study owner
		// when orders were provided the request migth be forwarded to the Group Orders or the Corrections page later
	
		if ( ! MergeStudies.checkStudyOwners( null, studies ).booleanValue() ) {
			throw new IMCommonGUIException( is,
					LocalizedMessage.getSimpleMessage( "EXCEPTION_CASES_MERGE_STUDIES_DIFFERENTE_SERVERS" ),
					tab, LocalizedMessage.getSimpleMessage( "EXCEPTION_CASES_MERGE_MERGINT_ERROR" ), back, 0 );
		}
		
		// Formerly the merge page was redirected to HUBs (from a CWL).
		// it no longer needed as doMerge() use rmiSubstitute to perform the merge on the owner servers 
	}


	MergeStudies merge = new MergeStudies( studies, is );

	if ( !merge.checkAccessList() )
		throw new IMCommonGUIException( is, LocalizedMessage.getSimpleMessage( "EXCEPTION_CASES_MERGE_NO_RIGHT_THESE_STUDIES" ), tab, LocalizedMessage.getSimpleMessage( "EXCEPTION_CASES_ORDERS_MERGE_TITLE" ), back, 0 );

	Vector<String> vOrders = new Vector<String>();
	Vector<String> vStudies = new Vector<String>();

	boolean isOK = merge.checkMerge( vOrders, vStudies );
	
	if ( isIface && vStudies.size() == 1 && vOrders.size() == 1 ) {
		// CBT #1219 : API command to access Merge Studies page
		// 4. If one of the Study UIDs is an EP order, the user shall be taken to the Corrections page.

		StringBuilder correctLink = new StringBuilder( 256 );
		
		correctLink.append( "/qc/confirm.jsp?cOrder=" ).append( vOrders.elementAt( 0 ) );
		correctLink.append( "&cStudy=" ).append( vStudies.elementAt( 0 ) );
		if ( back != null ) {
			correctLink.append( "&cBack=" ).append( Helper.URLEncodeUTF8( back ) );
		}
		correctLink.append( "&cIface=" ).append( parIface );
		response.sendRedirect( correctLink.toString() );
	}

%><%-- ----- HTML code ----- --%>
<HTML>
<HEAD><TITLE><%= is.http_server %> - <%= LocalizedMessage.getSimpleMessage( "CASES_MERGE_MERGE_STUDIES" ) %></TITLE>
<SCRIPT Language="JavaScript">
	function doSubmit(frm,val) {
		frm.cOp.value = val;
		frm.submit();
	}
</script>
<%
	Table.printCommonScript( is );
	merge.printHoverJS();
%>
</HEAD>
<BODY>
<%-- We need to place the form start tag here to avoid extra new line (MS IE stupidity) --%>
<FORM NAME=mergeForm ACTION="merge.jsp" METHOD="POST">
<CENTER>
<%
	if ( tab != null ) {
		TabToolbox.makeTabs( is, is.tabbean, tab, "White" );
	}

	if ( isOK && "Merge".equals( is.request.getParameter( "cOp" ) ) ) {
		if ( merge.doMerge() ) {
			out.println( "</FORM><BR>" );
			MessageBox mb = new MessageBox( LocalizedMessage.getSimpleMessage( "CASES_MERGE_MERGE_STARTED" ), "60%" );
			if ( back != null ) {
				mb.addButton( new Button( back, "BUTTON_BACK_TO_LIST", "ALT_TEXT_BACK" ) );
			}
			mb.setMessage( "<CENTER><h3>" + LocalizedMessage.getSimpleMessage( "CASES_MERGE_MERGIN_HAS_STARTED" ) + "</h3></CENTER>" );
			mb.print( is.out );
			out.println( "</center></body></html>" );
			return;
		}
		else {
			throw new IMCommonGUIException( is, merge.getErrorMessage(), tab, LocalizedMessage.getSimpleMessage( "EXCEPTION_CASES_ORDERS_MERGE_TITLE" ), back, 0 );
		}
	}

	boolean isInput = !"Confirm".equals( request.getParameter( "cOp" ) ) && !"Merge".equals( is.request.getParameter( "cOp" ) );
	String primaryStudy = request.getParameter( "cPrimary" );

	merge.printTable( primaryStudy, isInput );


	if ( !vOrders.isEmpty() ) {
		MessageBox mb = new MessageBox( LocalizedMessage.getSimpleMessage( "EXCEPTION_CASES_ORDERS_MERGE_TITLE" ), "50%" );
		if ( back != null ) {
			mb.addButton( new Button( back, "BUTTON_BACK_TO_LIST", "ALT_TEXT_BACK" ) );
		}

		String msg = "";

		if (isIface) {
			// From HTTP API only merging studies and correcting a single study to a single order is supported
			msg = LocalizedMessage.getSimpleMessage( "CASES_MERGE_HTTPAPI_CANNOT_MERGE" );
		}
		else if ( vStudies.size() == 1 && vOrders.size() == 1 ) {
			StringBuilder correctLink = new StringBuilder( 256 );
			
			correctLink.append( "/qc/confirm.jsp?cOrder=" ).append( vOrders.elementAt( 0 ) );
			correctLink.append( "&cStudy=" ).append( vStudies.elementAt( 0 ) );
			correctLink.append( "&cBack=" ).append( Helper.URLEncodeUTF8( back ) );
			
			Format[] msgFormat = new Format[] { null, null };
			Object[] msgObject = new Object[] { m_correctionsPageLink, correctLink.toString() };
			msg = LocalizedMessage.getCompoundMessage( "CASES_MERGE_YOU_CAN_MATCH_STUDY_AND_ORDER_MSG", msgFormat, msgObject, true);
		}
		else if ( vStudies.isEmpty() ) {
			String groupLink = "/cases/orders.jsp?cOper=merge&cBack=" + Helper.URLEncodeUTF8( back );
			for ( Iterator<String> iter = vOrders.iterator(); iter.hasNext(); ) {
				groupLink = groupLink + "&cGr=" + iter.next();
			}
			Format[] msgFormat = new Format[] { null, null };
			Object[] msgObject = new Object[] { m_groupPageLink, groupLink };
			msg = LocalizedMessage.getCompoundMessage( "CASES_MERGE_YOU_CAN_GROUP_ORDERS_MSG", msgFormat, msgObject, true); 
		}
		else {
			Format[] msgFormat = new Format[] { null, null };
			Object[] msgObject = new Object[] { m_groupPageLink, m_correctionsPageLink };
			msg = LocalizedMessage.getCompoundMessage( "CASES_MERGE_YOU_CAN_GROUP_ORDERS_OR_STUDY_AND_ORDER_MSG", msgFormat, msgObject, true);
		}

		mb.setMessage( "<TABLE><TR><TD><H3>" + LocalizedMessage.getSimpleMessage( "CASES_MERGE_ONLY_STUDIES_CAN_MERGE" ) + "<BR><BR>" + msg +
				"</H3></TD></TR></TABLE>" );

		out.println( "<BR>" );
		mb.print( is.out );
		out.println( "</FORM></CENTER></BODY></HTML>" );
		return;
	}

	if ( merge.printShortcutMessage() ) {
		// There were shortcuts selected to be merged and error dialog has been displayed
		out.println( "</CENTER></BODY></HTML>" );
		return;
	}


	if ( !isOK ) {
		// Either one of the studies is locked, or uneditable or ...
		// Print error message

		MessageBox mb = new MessageBox( LocalizedMessage.getSimpleMessage( "EXCEPTION_CASES_ORDERS_MERGE_TITLE" ), "50%" );
		if ( back != null ) {
			mb.addButton( new Button( back, "BUTTON_BACK_TO_LIST", "ALT_TEXT_BACK" ) );
		}
		boolean isPlural = merge.getNumberOfProblemStudies() > 1;
		String messageID = ( isPlural ? "CASES_MERGE_STUDY_CANT_MERGE" : "CASES_MERGE_STUDY_CANT_MERGE_PLURAL" );
		String msg = LocalizedMessage.getSimpleMessage( messageID );
		//String msg = ( isPlural ? "Some" : "One" )
		//		+ " of the selected studies can not be merged.<BR>For more information on the problem move the mouse over the red indicator light of the affected "
		//		+ ( isPlural ? "studies" : "study" );

		mb.setMessage( "<TABLE><TR><TD><H3><DIV>" + msg + "</DIV></H3></TD></TR></TABLE>" );

		out.println( "<BR>" );
		mb.print( is.out );
		out.println( "</FORM></CENTER></BODY></HTML>" );
		return;
	}
%>
<TABLE border=1 cellpadding=0 cellspacing=0 width="100%">
<%
	for ( int i = 0; i < studies.size(); i++ ) {
%>
 <INPUT TYPE="HIDDEN" NAME="cGr" VALUE="<%= Helper.htmlEncode( studies.elementAt( i ) ) %>">
<%
	}

	if ( null != request.getParameter( "cContext" ) ) {
%>
 <INPUT type="hidden" name="cContext" VALUE="<%=Helper.htmlEncode( request.getParameter( "cContext" ) )%>">
<%
	}
	
	String refererHost = request.getParameter( "RefererHost" );
	if ( refererHost != null ) {
%>
 <INPUT type="hidden" name="RefererHost" VALUE="<%=Helper.htmlEncode( refererHost ) %>">
<%
	}

	if ( isIface ) {
%>
 <INPUT type="hidden" name="cIface" VALUE="<%= parIface %>">
<%
	}
	
	if ( back != null ) {
%>
 <INPUT TYPE="HIDDEN" NAME="cBack" VALUE="<%=Helper.htmlEncode( back )%>">
<%
	}
%>
 <INPUT TYPE="HIDDEN" NAME="cOp">

<TR><TD colspan=5 class=BgShadow align=center><TABLE border=0 cellpadding=5><TR>
<TD colspan=2 align=center>
<BR><%= LocalizedMessage.getSimpleMessage( "CASES_MERGE_PATIENT_AND_STUDY_COPIED" ) %></TD></TR>

<%
	if ( isInput ) {
%>
<TR>
<TD><input type=checkbox name=cKeepReports value="true"	checked> <%=  LocalizedMessage.getSimpleMessage( "CASES_MERGE_KEEP_ALL_REPORTS" ) %></TD>
<TD><%= LocalizedMessage.getSimpleMessage( "CASES_MERGE_IF_CHECKED_IF_CLEARED" ) %></TD>
</TR>
<%
	}
	else if ( "true".equals( request.getParameter( "cKeepReports" ) ) ) {
%>
<TR><TD colspan=2 align=center>
<%= LocalizedMessage.getSimpleMessage( "CASES_MERGE_REPORTS_ATTACHED_ADDENDUMS" ) %>
<INPUT TYPE=hidden NAME="cKeepReports" value="true"></TD></TR>
<%
	}
	else {
%>
<TR><TD colspan=2 align=center><%= LocalizedMessage.getSimpleMessage( "CASES_MERGE_PURGE_REPORTS" ) %>
<INPUT TYPE=hidden NAME="cKeepReports" value="false"></TD></TR>
<%
	}
%>
<TD></TR></TABLE>
<%
	if ( !isInput ) {
%>
<BR><center><b><%= LocalizedMessage.getSimpleMessage( "CASES_MERGE_CLICK_ON_MERGE" ) %></b></center>
<%
	}
%>
</BR></TD></TR>
<%
	String action = isInput ? "Confirm" : "Merge";
	String cancel = back; //isInput ? back : "javascript:self.history.back();";
%>
<TR><TD colspan=5 class=BgShadow align=right><BR>
<a href="javascript:doSubmit(self.document.mergeForm, '<%= action %>');")>
	<%=  JSHelper.getJsLocalizedButton( "BUTTON_MERGE", "BUTTON_MERGE", false ) %>
</a>
<%
	if ( back != null ) {
%>
&nbsp;&nbsp;
<a href="<%= back %>">
	<%= ( JSHelper.getJsLocalizedButton( "BUTTON_CANCEL", "BUTTON_NULL", false ) ) %>
</a>
<%
	}
%>
&nbsp;&nbsp;</TD></TR>
<TR><TD colspan=5 class=BgShadow></TD></TR></TABLE>
</FORM>

</CENTER></BODY></HTML>